import * as i0 from '@angular/core';
import { Component, Input, NgModule } from '@angular/core';
import { messageFromError } from 'd2f-ngx-commons';
import * as i1 from '@angular/common';
import * as i2 from '@angular/forms';
import { FormsModule } from '@angular/forms';
import { throwError } from 'rxjs';
import { BrowserModule } from '@angular/platform-browser';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';

class GenericCrudComponent {
    constructor() {
        this.genericCrudContext = null;
        this.genericCrudService = null;
        //this.genericCrudContext?.tabObjects of type T[]
        this.idKeyName = "id"; //or "code" or "ref" or "num" or ...
        this.withAutoGeneratedId = false; // or true 
        //[(ngModel)]="deviseTemp.code" , ....
        this.objectTemp = null;
        this.message /*: string*/ = "";
        this.mode = "newOne";
    }
    cloneObject(obj) {
        return JSON.parse(JSON.stringify(obj));
    }
    copyObjectProperties(source, target) {
        let arrayOfPropKeys = Reflect.ownKeys(source);
        for (let key of arrayOfPropKeys) {
            Reflect.set(target, key, Reflect.get(source, key));
        }
    }
    objectKeysArray(obj) {
        return Reflect.ownKeys(obj);
    }
    essentialKeysArray() {
        return this.genericCrudContext.contextHelper.objectHelper().essentialFieldNames();
    }
    objectValuesArray(obj) {
        let arrayOfPropKeys = Reflect.ownKeys(obj);
        let valuesArray = [];
        for (let key of arrayOfPropKeys) {
            valuesArray.push(Reflect.get(obj, key));
        }
        return valuesArray;
    }
    objectEssentialValuesArray(obj) {
        let arrayOfPropKeys = this.essentialKeysArray();
        let valuesArray = [];
        for (let key of arrayOfPropKeys) {
            valuesArray.push(Reflect.get(obj, key));
        }
        return valuesArray;
    }
    isNoEditableId(attrName) {
        if (this.idKeyName != attrName)
            return false;
        /* else is key attr*/
        if (this.withAutoGeneratedId)
            return true;
        if (this.mode == 'existingOne')
            return true;
        return false;
    }
    ngOnInit() {
        var _a;
        if (this.genericCrudContext == null)
            return;
        this.idKeyName = this.genericCrudContext.contextHelper.objectHelper().getIdKeyName();
        this.withAutoGeneratedId = this.genericCrudContext.contextHelper.objectHelper().withAutoGeneratedId();
        this.genericCrudContext.onGetAllObjects$(this.genericCrudService)
            .subscribe({ next: (tabObjects) => {
                var _a;
                this.message = ((_a = this.genericCrudContext) === null || _a === void 0 ? void 0 : _a.entityTypeName) + " loaded";
                if (this.genericCrudContext)
                    this.genericCrudContext.tabObjects = tabObjects;
            },
            error: (err) => { this.message = messageFromError(err, "echec onGetAllObjects$"); } });
        this.objectTemp = (_a = this.genericCrudContext) === null || _a === void 0 ? void 0 : _a.contextHelper.objectHelper().buildEmptyObject();
    }
    onNew() {
        var _a;
        this.mode = "newOne";
        this.selectedObject = undefined;
        this.objectTemp = (_a = this.genericCrudContext) === null || _a === void 0 ? void 0 : _a.contextHelper.objectHelper().buildEmptyObject();
    }
    onAdd() {
        var _a;
        (_a = this.genericCrudService) === null || _a === void 0 ? void 0 : _a.postEntityObject$(this.objectTemp).subscribe({ next: (savedObject) => {
                var _a;
                this.message = ((_a = this.genericCrudContext) === null || _a === void 0 ? void 0 : _a.entityTypeName) + " added";
                this.addClientSide(savedObject);
            },
            error: (err) => { this.message = messageFromError(err, "echec post"); } });
    }
    addClientSide(savedObject) {
        var _a;
        (_a = this.genericCrudContext) === null || _a === void 0 ? void 0 : _a.tabObjects.push(savedObject);
        this.onNew();
    }
    onDelete() {
        var _a, _b;
        if (this.selectedObject) {
            let id = (_a = this.genericCrudContext) === null || _a === void 0 ? void 0 : _a.contextHelper.objectHelper().getId(this.selectedObject);
            (_b = this.genericCrudContext) === null || _b === void 0 ? void 0 : _b.onDeleteObject$(id, this.genericCrudService).subscribe({ next: () => {
                    var _a;
                    this.message = ((_a = this.genericCrudContext) === null || _a === void 0 ? void 0 : _a.entityTypeName) + " deleted";
                    this.deleteClientSide();
                },
                error: (err) => { this.message = messageFromError(err, "echec suppression"); } });
        }
    }
    deleteClientSide() {
        var _a, _b;
        if (this.selectedObject) {
            let indexToDelete = -1;
            (_a = this.genericCrudContext) === null || _a === void 0 ? void 0 : _a.tabObjects.forEach((obj, index) => {
                if (obj == this.selectedObject)
                    indexToDelete = index;
            });
            if (indexToDelete >= 0) {
                (_b = this.genericCrudContext) === null || _b === void 0 ? void 0 : _b.tabObjects.splice(indexToDelete, 1);
            }
        }
        this.onNew();
    }
    onUpdate() {
        var _a;
        (_a = this.genericCrudContext) === null || _a === void 0 ? void 0 : _a.onUpdateObject$(this.objectTemp, this.genericCrudService).subscribe({ next: (updatedObject) => {
                var _a;
                this.message = ((_a = this.genericCrudContext) === null || _a === void 0 ? void 0 : _a.entityTypeName) + " updated";
                this.updateClientSide(updatedObject);
            },
            error: (err) => { this.message = messageFromError(err, "echec update (put)"); } });
    }
    updateClientSide(updatedObject) {
        //test imposé par typescript sur this.selectedObject potentiellement undefined
        if (this.selectedObject != undefined) {
            //Rappel: this.selectedObject est ici une référence
            //qui pointe directement sur le i eme objet du tableau this.tabObjects
            //(selon ligne sélectionnée)
            this.copyObjectProperties(updatedObject, this.selectedObject);
        }
    }
    //fonction évenementielle à appeler lorsque l'on
    //va sélectionner une des lignes du tableau
    onSelectObject(o) {
        var _a;
        //NB: o:any est passé par référence (comportement de java/javascript)
        //et donc ici o et this.selectedObject référencent
        //directement un des objets du tableau this.tabObjects
        this.selectedObject = o;
        this.mode = "existingOne";
        //via un clonage explicite , this.objectTemp est une copie
        //indépendante de this.selectedObject (et pas une référence sur l'objet original)
        this.objectTemp = this.cloneObject(this.selectedObject);
        let id = (_a = this.genericCrudContext) === null || _a === void 0 ? void 0 : _a.contextHelper.objectHelper().getId(this.selectedObject);
        this.message = id + " selected";
    }
}
GenericCrudComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.1.0", ngImport: i0, type: GenericCrudComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
GenericCrudComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "14.1.0", type: GenericCrudComponent, selector: "generic-crud", inputs: { genericCrudContext: "genericCrudContext", genericCrudService: "genericCrudService" }, ngImport: i0, template: "<div class=\"flexRowWrap mb-1 mt-1\">\n    <div class=\"flexItem m-1\" >\n      <div class=\"mat-card\">\n       <div class=\"mat-card-header text-white bg-primary\">List of {{genericCrudContext!.entityTypeName}}</div>\n       <div class=\"mat-card-content  p-1\">\n       <div class=\"tableContainerWithFixHead\">\n       <table class=\"table\" border=\"1\">\n          <thead>\n           <tr>\n              <th *ngFor=\"let key of  essentialKeysArray()\">{{key}}</th>\n            </tr>\n        </thead>\n        <tbody>\n           <tr *ngFor=\"let d of genericCrudContext!.tabObjects ; let i=index\"\n           (click)=\"onSelectObject(d)\"\n           [class.selected]=\"d==selectedObject\">\n           <td *ngFor=\"let val of objectEssentialValuesArray(d)\">{{val}}</td>\n          </tr>\n        </tbody>\n       </table> <!-- end of table -->\n      </div><!-- end of tableContainerWithFixHead -->\n       <b>{{message}}</b>\n       </div> <!-- end of mat-card-content -->\n      </div><!-- end of mat-card -->\n     </div> <!-- end of flexItem -->\n     <div class=\"flexItem m-1\" >\n       <div class=\"mat-card\">\n       <div class=\"mat-card-header text-white bg-primary\">selected {{genericCrudContext!.entityTypeName}} ({{mode}})</div>\n       <div class=\"mat-card-content p-1\">\n       <form #formObject=\"ngForm\">  \n           <div class=\"flexRowWrap\" *ngFor=\"let key of objectKeysArray(objectTemp)\" > \n           <label class=\"f-align\">{{key}}:</label>\n           <div class=\"f-align\">\n               <input *ngIf=\"!isNoEditableId(key)\" [name]=\"key\" class=\"f-max-size  mb-1\" [(ngModel)]=\"objectTemp[key]\" required />\n               <label *ngIf=\"isNoEditableId(key)\"   class=\"f-max-size  mb-1\">{{objectTemp[key]}}</label>\n           </div>       \n           </div><!-- end of flexRowWrap -->\n           <div class=\"flexRowWrap\">\n               <div class=\"f-align\">\n                    <button (click)=\"onUpdate()\" class=\"btn-primary\" \n                        [style.visibility]=\"(mode=='existingOne')?'visible':'hidden'\"\n                        [disabled]=\"!formObject.form.valid\"  >\n                        update <i class=\"bi-save\"></i></button> &nbsp;\n                    <button (click)=\"onNew()\" class=\"btn-primary\"> new </button>  &nbsp;\n                    <button (click)=\"onAdd()\" [disabled]=\"!formObject.form.valid\"\n                            [style.visibility]=\"(mode=='newOne')?'visible':'hidden'\" \n                            class=\"btn-primary\"> add </button> &nbsp;\n                    <button (click)=\"onDelete()\" [style.visibility]=\"(mode=='existingOne')?'visible':'hidden'\" class=\"btn-danger\"> delete </button>\n               </div> \n           </div><!-- end of flexRowWrap -->\n       </form><!-- end of form -->\n       </div> <!-- end of mat-card-content -->\n      </div><!-- end of mat-card -->\n     </div> <!-- end of flexItem -->\n   </div><!-- end of main flexRowNoWrap -->\n", styles: ["@import\"/assets/css/d2f-common-layout.css\";.flexRowWrap{align-items:stretch!important}.tableContainerWithFixHead{overflow:auto;height:100px}.tableContainerWithFixHead th{position:sticky;top:0}table{border-collapse:collapse;width:100%}th,td{padding:1px}th{background:#eee;text-align:left}.selected{color:#00f;font-weight:700}.mat-card-header{color:#fff!important;background-color:#00f!important}label.f-align{width:8em;display:inline-block;margin-left:.3em;flex:0 0 8em}.f-align{margin-left:.3em;flex:1 1 auto}.f-max-size{width:97%}\n"], dependencies: [{ kind: "directive", type: i1.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "directive", type: i1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "directive", type: i2.ɵNgNoValidate, selector: "form:not([ngNoForm]):not([ngNativeValidate])" }, { kind: "directive", type: i2.DefaultValueAccessor, selector: "input:not([type=checkbox])[formControlName],textarea[formControlName],input:not([type=checkbox])[formControl],textarea[formControl],input:not([type=checkbox])[ngModel],textarea[ngModel],[ngDefaultControl]" }, { kind: "directive", type: i2.NgControlStatus, selector: "[formControlName],[ngModel],[formControl]" }, { kind: "directive", type: i2.NgControlStatusGroup, selector: "[formGroupName],[formArrayName],[ngModelGroup],[formGroup],form:not([ngNoForm]),[ngForm]" }, { kind: "directive", type: i2.RequiredValidator, selector: ":not([type=checkbox])[required][formControlName],:not([type=checkbox])[required][formControl],:not([type=checkbox])[required][ngModel]", inputs: ["required"] }, { kind: "directive", type: i2.NgModel, selector: "[ngModel]:not([formControlName]):not([formControl])", inputs: ["name", "disabled", "ngModel", "ngModelOptions"], outputs: ["ngModelChange"], exportAs: ["ngModel"] }, { kind: "directive", type: i2.NgForm, selector: "form:not([ngNoForm]):not([formGroup]),ng-form,[ngForm]", inputs: ["ngFormOptions"], outputs: ["ngSubmit"], exportAs: ["ngForm"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.1.0", ngImport: i0, type: GenericCrudComponent, decorators: [{
            type: Component,
            args: [{ selector: 'generic-crud', template: "<div class=\"flexRowWrap mb-1 mt-1\">\n    <div class=\"flexItem m-1\" >\n      <div class=\"mat-card\">\n       <div class=\"mat-card-header text-white bg-primary\">List of {{genericCrudContext!.entityTypeName}}</div>\n       <div class=\"mat-card-content  p-1\">\n       <div class=\"tableContainerWithFixHead\">\n       <table class=\"table\" border=\"1\">\n          <thead>\n           <tr>\n              <th *ngFor=\"let key of  essentialKeysArray()\">{{key}}</th>\n            </tr>\n        </thead>\n        <tbody>\n           <tr *ngFor=\"let d of genericCrudContext!.tabObjects ; let i=index\"\n           (click)=\"onSelectObject(d)\"\n           [class.selected]=\"d==selectedObject\">\n           <td *ngFor=\"let val of objectEssentialValuesArray(d)\">{{val}}</td>\n          </tr>\n        </tbody>\n       </table> <!-- end of table -->\n      </div><!-- end of tableContainerWithFixHead -->\n       <b>{{message}}</b>\n       </div> <!-- end of mat-card-content -->\n      </div><!-- end of mat-card -->\n     </div> <!-- end of flexItem -->\n     <div class=\"flexItem m-1\" >\n       <div class=\"mat-card\">\n       <div class=\"mat-card-header text-white bg-primary\">selected {{genericCrudContext!.entityTypeName}} ({{mode}})</div>\n       <div class=\"mat-card-content p-1\">\n       <form #formObject=\"ngForm\">  \n           <div class=\"flexRowWrap\" *ngFor=\"let key of objectKeysArray(objectTemp)\" > \n           <label class=\"f-align\">{{key}}:</label>\n           <div class=\"f-align\">\n               <input *ngIf=\"!isNoEditableId(key)\" [name]=\"key\" class=\"f-max-size  mb-1\" [(ngModel)]=\"objectTemp[key]\" required />\n               <label *ngIf=\"isNoEditableId(key)\"   class=\"f-max-size  mb-1\">{{objectTemp[key]}}</label>\n           </div>       \n           </div><!-- end of flexRowWrap -->\n           <div class=\"flexRowWrap\">\n               <div class=\"f-align\">\n                    <button (click)=\"onUpdate()\" class=\"btn-primary\" \n                        [style.visibility]=\"(mode=='existingOne')?'visible':'hidden'\"\n                        [disabled]=\"!formObject.form.valid\"  >\n                        update <i class=\"bi-save\"></i></button> &nbsp;\n                    <button (click)=\"onNew()\" class=\"btn-primary\"> new </button>  &nbsp;\n                    <button (click)=\"onAdd()\" [disabled]=\"!formObject.form.valid\"\n                            [style.visibility]=\"(mode=='newOne')?'visible':'hidden'\" \n                            class=\"btn-primary\"> add </button> &nbsp;\n                    <button (click)=\"onDelete()\" [style.visibility]=\"(mode=='existingOne')?'visible':'hidden'\" class=\"btn-danger\"> delete </button>\n               </div> \n           </div><!-- end of flexRowWrap -->\n       </form><!-- end of form -->\n       </div> <!-- end of mat-card-content -->\n      </div><!-- end of mat-card -->\n     </div> <!-- end of flexItem -->\n   </div><!-- end of main flexRowNoWrap -->\n", styles: ["@import\"/assets/css/d2f-common-layout.css\";.flexRowWrap{align-items:stretch!important}.tableContainerWithFixHead{overflow:auto;height:100px}.tableContainerWithFixHead th{position:sticky;top:0}table{border-collapse:collapse;width:100%}th,td{padding:1px}th{background:#eee;text-align:left}.selected{color:#00f;font-weight:700}.mat-card-header{color:#fff!important;background-color:#00f!important}label.f-align{width:8em;display:inline-block;margin-left:.3em;flex:0 0 8em}.f-align{margin-left:.3em;flex:1 1 auto}.f-max-size{width:97%}\n"] }]
        }], ctorParameters: function () { return []; }, propDecorators: { genericCrudContext: [{
                type: Input
            }], genericCrudService: [{
                type: Input
            }] } });

class GenericCrudContext {
    constructor(contextHelper) {
        this.contextHelper = contextHelper;
        this.entityTypeName = "UnknownEntityName"; //ex: "Devise" ou "Contact" ou ...
        this.tabObjects = [];
        this.entityTypeName = contextHelper.objectHelper().getEntityTypeName();
    }
    onGetAllObjects$(genericCrudService) {
        let crudHelper = this.contextHelper.crudHelper();
        if (crudHelper != null)
            return crudHelper.onGetAllObjects$();
        else if (genericCrudService)
            return genericCrudService.getAllObjects$(null);
        //<Observable<T[]>> <any> = temporal workaround for unknown angular library version mismatch (rxjs from d2f-ngx-commons) when npm link (no problem with npm i -s d2f-ngx-commons)
        else
            return throwError(() => { err: "onGetAllObjects$ not implemented (no genericCrudService and no crudHelper)"; });
    }
    onAddObject$(obj, genericCrudService) {
        let crudHelper = this.contextHelper.crudHelper();
        if (crudHelper != null)
            return crudHelper.onAddObject$(obj);
        else if (genericCrudService)
            return genericCrudService.postEntityObject$(obj);
        else
            return throwError(() => { err: "onAddObject$ not implemented (no genericCrudService and no crudHelper)"; });
    }
    onUpdateObject$(obj, genericCrudService) {
        let crudHelper = this.contextHelper.crudHelper();
        if (crudHelper != null)
            return crudHelper.onUpdateObject$(obj);
        else if (genericCrudService)
            return genericCrudService.putEntityObject$(obj);
        else
            return throwError(() => { err: "onUpdateObject$ not implemented (no genericCrudService and no crudHelper)"; });
    }
    onDeleteObject$(id, genericCrudService) {
        let crudHelper = this.contextHelper.crudHelper();
        if (crudHelper != null)
            return crudHelper.onDeleteObject$(id);
        else if (genericCrudService)
            return genericCrudService.deleteEntityObjectServerSide$(id);
        else
            return throwError(() => { err: "onDeleteObject$ not implemented (no genericCrudService and no crudHelper)"; });
    }
}

class D2fNgxCrudModule {
}
D2fNgxCrudModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.1.0", ngImport: i0, type: D2fNgxCrudModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
D2fNgxCrudModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "14.1.0", ngImport: i0, type: D2fNgxCrudModule, declarations: [GenericCrudComponent], imports: [BrowserModule,
        BrowserAnimationsModule,
        FormsModule], exports: [GenericCrudComponent] });
D2fNgxCrudModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "14.1.0", ngImport: i0, type: D2fNgxCrudModule, imports: [BrowserModule,
        BrowserAnimationsModule,
        FormsModule] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.1.0", ngImport: i0, type: D2fNgxCrudModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [
                        GenericCrudComponent
                    ],
                    imports: [
                        BrowserModule,
                        BrowserAnimationsModule,
                        FormsModule
                    ],
                    exports: [
                        GenericCrudComponent
                    ]
                }]
        }] });

/*
 * Public API Surface of d2f-ngx-crud
 */

/**
 * Generated bundle index. Do not edit.
 */

export { D2fNgxCrudModule, GenericCrudComponent, GenericCrudContext };
//# sourceMappingURL=d2f-ngx-crud.mjs.map
